require('config')
require('toggle')
require('button')
require('number')

group_option = {}

function group_option:new(name, column, parent)
  local o = setmetatable({}, { __index = self })
  table.insert(parent, o)

  o.name = name
  o.column = column
  o.options = {}

  return o
end

function group_option:toggle(name)
  local toggle = toggle_option:new(name)
  table.insert(self.options, toggle)
  return toggle
end

function group_option:button(name, on_press)
  local button = button_option:new(name, on_press)
  table.insert(self.options, button)
  return button
end

function group_option:number(name, fmt, step, min, max, value)
  local number = number_option:new(name, fmt, step, min, max, value)
  table.insert(self.options, number)
  return number
end

function group_option:render(y, content_position, content_scale, padding)
  local height = 0.0
  for i, option in ipairs(self.options) do
    if i == 1 then
      height = height + (padding * 2.0)
    end

    height = height + option:get_height() + (padding * 2.0)
  end

  local position = vec2()
  if self.column == 1 then
    position = vec2(content_position.x + padding, y + padding)
  else
    position = vec2(content_position.x + content_scale.x * 0.5 + padding, y + padding)
  end

  -- title
  gui.text(self.name)
    :position(position)
    :color(config.group.text)
    :override_scale(config.group.scale)
    :draw()

  position.y = position.y + config.group.scale

  -- banner
  gui.rect(position, vec2(content_scale.x * 0.5 - padding * 2.0, config.group.banner_height))
    :flags(config.group.banner_flags)
    :multicolors(config.group.banner_ul, config.group.banner_ur, config.group.banner_br, config.group.banner_bl)
    :draw()

  position.y = position.y + config.group.banner_height

  -- background
  gui.rect(position, vec2(content_scale.x * 0.5 - padding * 2.0, height))
    :rounding(config.group.rounding)
    :flags(config.group.bg_flags)
    :color(config.group.bg)
    :draw()

  local render_position = vec2(position.x + (padding * 2.0), position.y + (padding * 2.0))
  for i, option in ipairs(self.options) do
    render_position.y = render_position.y + option:render(render_position, content_scale.x * 0.5 - ((padding * 2.0) * 2.0) - (padding * 2.0), (padding * 2.0)) + (padding * 2.0)
  end

  return height + config.group.scale + config.group.banner_height + config.group.spacing
end