require('scrollbar')
require('group')
require('table')
require('config')

local native = require('natives')
local snow = require('snow')
local snow_effect = snow:new()

create = {
  player_root = function()
    local main = group_option:new('Main', Column.LEFT, options.player.root)
    
    options.player.invincible = main:toggle('Invincible')
    options.player.never_wanted = main:toggle('Never Wanted')
  
    main:button('Notify', function()
      notify.push('Lua Script', 'Very nice script, keep it up')
    end)
  end,

  settings_root = function()
    local bg = group_option:new('Background', Column.LEFT, options.settings.root)

    bg:number('Height', '%.2f', 1.0, 0.0, 1000.0, config.main.scale.y)
      :on_change(function(opt)
        config.main.scale.y = opt.value
      end)

    bg:number('Rounding', '%.2f', 1.0, 0.0, 10.0, config.main.bg_rounding)
      :on_change(function(opt)
        config.main.bg_rounding = opt.value
      end)

    bg:number('Thickness', '%.2f', 1.0, 0.0, 10.0, config.main.bg_thickness)
      :on_change(function(opt)
        config.main.bg_thickness = opt.value
      end)
  end
}

create.player_root()
create.settings_root()

events.subscribe(Event.SCROLL_WHEEL, function(data)
  config.tab_scrollbar:update(data.offset)
end)

-- Feature thread
util.create_thread(function()
  if game.state() ~= GameState.NONE then
    return -- Only run these options when you're in a game
  end

  if options.player.invincible.toggle then
    native.set_entity_invincible(players.me().ped, true)
  end

  if options.player.never_wanted.toggle then
    native.set_max_wanted_level(0)
    native.clear_player_wanted_level(players.me().id)
  end
end)

-- UI thread
util.create_thread(function()
  snow_effect:update()
  snow_effect:render()

  input.cursor(config.is_open)

  if not config.is_open then
    return
  end

  -- background
  gui.rect(config.main.position, config.main.scale)
    :rounding(config.main.bg_rounding)
    :thickness(config.main.bg_thickness)
    :color(config.main.bg)
    :outline(config.main.bg_outline)
    :draw()

  -- tab line seperator
  gui.rect(vec2(config.main.position.x + config.main.tab_seperator, config.main.position.y), vec2(config.main.position.x + config.main.tab_seperator, config.main.position.y + config.main.scale.y))
    :line()
    :color(config.main.tab_line)
    :thickness(config.main.tab_line_thickness)
    :draw()

  -- sub tab seperator
  gui.rect(vec2(config.main.position.x + config.main.sub_tab_seperator, config.main.position.y), vec2(config.main.position.x + config.main.sub_tab_seperator, config.main.position.y + config.main.scale.y))
    :line()
    :color(config.main.sub_tab)
    :thickness(config.main.sub_tab_thickness)
    :draw()

  -- tabs on the left
  local tab_item_size = gui.get_size(config.tabs[1][1], config.main.tab_icon_size)
  local tab_item_size_total = tab_item_size + (config.main.tab_icon_box_size * 0.5)
  local tab_item_gap = ((config.main.tab_seperator - tab_item_size_total.x) * 0.5)
  
  for index, value in ipairs(config.tabs) do
    local icon = value[1]
    local name = value[2]
    local text_position = vec2(config.main.position.x + (config.main.tab_seperator * 0.5), config.main.position.y + config.main.tab_item_padding + 2.0 + ((tab_item_size_total.y + config.main.tab_item_padding + 2.0) * (index - 1)))
    
    local rect_color = config.main.tab_bg
    local icon_color = config.main.tab_icon

    if index == config.selected_tab then
      rect_color = config.main.selected_tab_bg
      icon_color = config.main.selected_tab_icon

      gui.rect(vec2(config.main.position.x + config.main.tab_seperator - 1.0, text_position.y + (tab_item_size_total.y * 0.5) - 5.0), config.main.selected_tab_bar_scale)
        :filled()
        :color(config.main.selected_tab_bar)
        :draw()
    end

    gui.rect(vec2(config.main.position.x + tab_item_gap, text_position.y), tab_item_size_total)
      :rounding(config.main.tab_rounding)
      :color(rect_color)
      :draw()

    gui.text(icon)
      :position(vec2(text_position.x, text_position.y + config.main.tab_item_padding + 2.0))
      :color(icon_color)
      :override_scale(config.main.tab_icon_size)
      :justify(Justify.CENTER)
      :draw()

    if input.is_mouse_in_bounds(vec2(config.main.position.x + tab_item_gap, text_position.y), tab_item_size_total) and input.mouse(1).just_released then
      config.selected_sub_tab = 1
      config.selected_tab = index
    end
  end
  
  -- sub tab title
  local sub_tab_total_width = (config.main.sub_tab_seperator - config.main.tab_seperator) - (tab_item_gap * 2.0)
  local sub_tab_title_position = vec2(config.main.position.x + config.main.tab_seperator + tab_item_gap + 1.0, config.main.position.y + config.main.tab_item_padding + 2.0)

  gui.rect(sub_tab_title_position, vec2(sub_tab_total_width, tab_item_size_total.y))
    :rounding(config.main.sub_tab_rounding)
    :color(config.main.sub_tab_title_bg)
    :draw()

  gui.text(config.tabs[config.selected_tab][1])
    :position(vec2(sub_tab_title_position.x + config.main.sub_tab_text_padding, sub_tab_title_position.y + config.main.tab_item_padding + 2.0))
    :color(config.main.sub_tab_title_icon)
    :override_scale(config.main.tab_icon_size)
    :draw()

  gui.text(config.tabs[config.selected_tab][2])
    :position(vec2(sub_tab_title_position.x + config.main.sub_tab_text_padding + config.main.tab_icon_size + config.main.tab_item_padding + 2.0, sub_tab_title_position.y + config.main.sub_tab_text_padding))
    :color(config.main.sub_tab_title_text)
    :override_scale(config.main.sub_tab_title_text_scale)
    :draw()

  for index, value in ipairs(config.sub_tabs[config.selected_tab]) do
    local rect_color = config.main.sub_tab_bg
    local text_color = config.main.sub_tab_text

    if index == config.selected_sub_tab then
      rect_color = config.main.selected_sub_tab_bg
      text_color = config.main.selected_sub_tab_text

      gui.rect(vec2(config.main.position.x + config.main.sub_tab_seperator - 1.0, sub_tab_title_position.y + (index * (tab_item_size_total.y + config.main.tab_item_padding + 2.0)) + (tab_item_size_total.y * 0.5) - 5.0), config.main.selected_sub_tab_bar_scale)
        :filled()
        :color(config.main.selected_sub_tab_bar)
        :draw()
    end

    gui.rect(vec2(sub_tab_title_position.x, sub_tab_title_position.y + (index * (tab_item_size_total.y + config.main.tab_item_padding + 2.0))), vec2(sub_tab_total_width, tab_item_size_total.y))
      :rounding(config.main.sub_tab_rounding)
      :color(rect_color)
      :draw()

    gui.text(config.sub_tabs[config.selected_tab][index][1])
      :position(vec2(sub_tab_title_position.x + config.main.tab_item_padding + 2.0, sub_tab_title_position.y + (index * (tab_item_size_total.y + config.main.tab_item_padding + 2.0) + 6.0)))
      :color(text_color)
      :override_scale(config.main.sub_tab_text_scale)
      :draw()

    if input.is_mouse_in_bounds(vec2(sub_tab_title_position.x, sub_tab_title_position.y + (index * (tab_item_size_total.y + config.main.tab_item_padding + 2.0))), vec2(sub_tab_total_width, tab_item_size_total.y)) and input.mouse(1).just_released then
      config.selected_sub_tab = index
    end
  end

  -- options
  local content_position = vec2(config.main.position.x + config.main.sub_tab_seperator + config.main.tab_item_padding, config.main.position.y + config.main.tab_item_padding)
  local content_scale = vec2(config.main.scale.x - config.main.sub_tab_seperator - (config.main.tab_item_padding * 2.0), config.main.scale.y - (config.main.tab_item_padding * 2.0))
  local content_height = 0.0
  
  gui.push_clip(content_position, content_scale)

  local option_heights = { 0, 0 } -- two columns

  local options = config.sub_tabs[config.selected_tab][config.selected_sub_tab][2]
  for index, option in ipairs(options) do
    local column = option.column
    local render_position = (content_position.y + option_heights[column]) - config.tab_scrollbar.target
    option_heights[column] = option_heights[column] + option:render(render_position, content_position, vec2(content_scale.x - tab_item_gap, content_scale.y), tab_item_gap)
  end

  if option_heights[1] > option_heights[2] then
    content_height = option_heights[1]
  else
    content_height = option_heights[2]
  end

  content_height = content_height + (config.main.tab_item_padding * 3.0)

  if content_height > content_scale.y then
    config.tab_scrollbar.rendering = true
    config.tab_scrollbar.vertical = true
    config.tab_scrollbar.position = vec2((content_position.x + content_scale.x) - config.main.scrollbar_width, content_position.y)
    config.tab_scrollbar.scale = vec2(config.main.scrollbar_width, content_scale.y)
    config.tab_scrollbar.unclipped_size = content_height
    config.tab_scrollbar.rounding = config.main.scrollbar_rounding
    config.tab_scrollbar.background_color = config.main.scrollbar_bg
    config.tab_scrollbar.bar_color = config.main.scrollbar_bar
    config.tab_scrollbar:draw(true)
  else
    config.tab_scrollbar.target = 0.0
    config.tab_scrollbar.rendering = false
  end

  gui.pop_clip()
end)