require('config')

number_option = {}

function number_option:new(name, fmt, step, min, max, value)
  value = value or 0.0
  local o = setmetatable({}, { __index = self })

  o.name = name
  o.fmt = fmt
  o.step = step
  o.min = min
  o.max = max
  o.value = value
  o.toggle = false
  o.scrollable = false
  o._on_change = function(opt) end

  return o
end

function number_option:on_change(func)
  self._on_change = func
  return self
end

function number_option:get_height()
  return config.number.bar_height + config.number.scale + config.number.bar_spacing
end

function number_option:render(pos, group_width, padding)
  gui.text(self.name)
    :position(pos)
    :color(config.number.text)
    :override_scale(config.number.scale)
    :draw()

  -- Background bar
  gui.rect(vec2(pos.x, pos.y + config.number.scale + config.number.bar_spacing), vec2(group_width, config.number.bar_height))
    :rounding(config.number.bar_rounding)
    :color(config.number.bar)
    :draw()

  -- Selected bar
  if self.value > self.min then
    local total_progress = self.max - self.min
    local current_progress = self.value - self.min
    local bar_width = utils.clamp((current_progress / total_progress) * group_width, 0.0, group_width)

    gui.rect(vec2(pos.x, pos.y + config.number.scale + config.number.bar_spacing), vec2(bar_width, config.number.bar_height))
      :rounding(config.number.bar_rounding)
      :color(config.number.bar_value)
      :draw()
  end

  -- Value
  gui.text(string.format(self.fmt, self.value))
    :position(vec2(pos.x + (group_width * 0.5), pos.y + config.number.scale + config.number.bar_spacing - 2.0))
    :color(config.number.text)
    :override_scale(config.number.bar_height + 4.0)
    :justify(Justify.CENTER)
    :draw()

  -- Input
  if self.scrollable or input.is_mouse_in_bounds(pos, vec2(group_width, self:get_height())) then
    if input.mouse(1).just_pressed then
      self.scrollable = true
    end

    if input.mouse(1).pressed and self.scrollable then
      local value = self.min + (self.max - self.min) * ((input.mouse_position().x - pos.x) / group_width)
      self.value = utils.clamp(value, self.min, self.max)

      self._on_change(self)
    else
      self.scrollable = false
    end
  end

  return self:get_height()
end