require('util')

scrollbar = {}

function scrollbar:new(o)
  o = o or {}
  setmetatable(o, self)
  self.__index = self

  self.target = 0.0
  self.hovered = false
  
  self.position = vec2()
  self.scale = vec2()
  self.last_cursor = vec2()
  self.capped_position = vec2()
  self.capped_scale = vec2()

  self.unclipped_size = 0.0
  self.last_unclipped_size = 0.0
  self.bar_offset = 0.0
  self.rounding = 0.0
  
  self.render_background = false
  self.vertical = false
  self.scrollable = false
  self.set_last_cursor = false
  self.rendering = false
  self.capped = false
  self.update_value = false
  
  self.background_color = color(0,0,0,0)
  self.bar_color = color(0,0,0,0)

  return o
end

function scrollbar:update(offset)
  if self.capped then
    if not input.is_mouse_in_bounds(self.capped_position, self.capped_scale) then
      return
    end
  end

  if self.rendering then
    local bar_size = 0.0

    if self.vertical then
      bar_size = self.scale.y / (self.unclipped_size / self.scale.y)
    else
      bar_size = self.scale.x / (self.unclipped_size / self.scale.x)
    end

    if offset < 0.0 then
      self.bar_offset = self.bar_offset - (offset + (bar_size * 0.25))
    else
      self.bar_offset = self.bar_offset + (offset + (bar_size * 0.25))
    end

    local clamp_value = 0.0
    if self.vertical then
      clamp_value = self.scale.y - bar_size
    else
      clamp_value = self.scale.x - bar_size
    end

    self.bar_offset = utils.clamp(self.bar_offset, 0.0, clamp_value)
    self.update_value = true
  end
end

function scrollbar:draw(draw_capped)
  self.capped = draw_capped

  if self.last_unclipped_size ~= self.unclipped_size then
    self.bar_offset = (self.scale.y / (self.unclipped_size / self.scale.y)) / ((self.scale.y / (self.last_unclipped_size / self.scale.y)) / self.bar_offset)
    self.last_unclipped_size = self.unclipped_size
  end

  local pos = vec2(self.position.x, self.position.y)
  local scale = vec2(self.scale.x, self.scale.y)
  local rendering_pos = vec2()
  local rendering_scale = vec2()
  local bar_size = 0.0

  gui.rect(pos, scale)
    :rounding(self.rounding)
    :color(self.background_color)
    :draw()

  if self.vertical then
    bar_size = self.scale.y / (self.unclipped_size / self.scale.y)
    rendering_pos = vec2(pos.x, pos.y + self.bar_offset)
    rendering_scale = vec2(scale.x, bar_size)
  else
    bar_size = self.scale.x / (self.unclipped_size / self.scale.x)
    rendering_pos = vec2(pos.x + self.bar_offset, pos.y)
    rendering_scale = vec2(bar_size, scale.y)
  end

  rendering_pos.x = rendering_pos.x + (self.scale.x * 0.2)
  rendering_pos.y = rendering_pos.y + (self.scale.x * 0.3)
  rendering_scale.x = rendering_scale.x - (self.scale.x * 0.4)
  rendering_scale.y = rendering_scale.y - (self.scale.x * 0.6)

  gui.rect(rendering_pos, rendering_scale)
    :rounding(self.rounding)
    :color(self.bar_color)
    :draw()
  
  if input.is_mouse_in_bounds(rendering_pos, rendering_scale) or self.scrollable then
    self.hovered = true

    if input.mouse(1).just_pressed then
      self.scrollable = true
    end

    if input.mouse(1).pressed and self.scrollable then
      if not self.set_last_cursor then
        self.set_last_cursor = true
        self.last_cursor = input.mouse_position()
      end

      if self.vertical then
        self.bar_offset = self.bar_offset + (input.mouse_position().y - self.last_cursor.y)
      else
        self.bar_offset = self.bar_offset + (input.mouse_position().x - self.last_cursor.x)
      end

      self.update_value = true
      self.last_cursor = input.mouse_position()
    else
      self.set_last_cursor = false
      self.scrollable = false
    end
  else
    self.hovered = false
  end

  if self.update_value then
    local clamp_value = 0.0
    if self.vertical then
      clamp_value = self.scale.y - bar_size
    else
      clamp_value = self.scale.x - bar_size
    end

    self.bar_offset = utils.clamp(self.bar_offset, 0.0, clamp_value)
    
    local clamp_value2 = 0.0
    if self.vertical then
      clamp_value2 = self.scale.y
    else
      clamp_value2 = self.scale.x
    end

    self.target = (self.bar_offset / clamp_value) * (self.unclipped_size - clamp_value2)
    self.target = utils.clamp(self.target, 0.0, self.unclipped_size - clamp_value2)
  end

  self.update_value = false
end