local snow = {}
local snow_settings = {
  position = vec2(0.0, 0.0),
  scale = game.resolution(),
  scale_range = vec2(0.5, 2.0),
  speed_range = vec2(5.0, 20.0),
  delay_range = vec2(0.0, 20.0),
  opacity_range = vec2(150.0, 255.0),
  offset_range = vec2(-20.0, 20.0),
  max_particles = 300,
  point_segments = 1
}

function snow:new()
  return setmetatable({animators = {}}, {__index = self})
end

function snow:render()
  for _, animator in ipairs(self.animators) do
    if animator:valid() then
      local cur = animator.current_frame
      gui.circle(cur.position, cur.scale, color(255, 255, 255, cur.opacity), snow_settings.point_segments)
    end
  end
end

function snow:reset()
  self.animators = {}
end

function snow:update()
  if #self.animators == 0 then
    for _ = 1, snow_settings.max_particles do
      self:create()
    end
  end

  for _, animator in ipairs(self.animators) do
    animator:update(game.delta())
  end
end

function snow:create()
  local anim = gui.animator()
  local start_position = snow_settings.position + vec2(util.random_float(1.0, 100.0) * snow_settings.scale.x / 100.0, -50.0)
  local offset = util.random_float(snow_settings.offset_range.x, snow_settings.offset_range.y) * snow_settings.scale.x / 100.0
  local y_percentage = util.random_float(30.0, 70.0)

  anim:frame(0.0).position = start_position
  anim:frame(0.0).opacity = util.random_float(snow_settings.opacity_range.x, snow_settings.opacity_range.y)
  anim:frame(0.0).scale = util.random_float(snow_settings.scale_range.x, snow_settings.scale_range.y)

  anim:frame(y_percentage).position = vec2(start_position.x + offset * 0.5, snow_settings.position.y + (y_percentage * snow_settings.scale.y / 100.0))
  anim:frame(y_percentage).opacity = anim:frame(0.0).opacity
  anim:frame(y_percentage).scale = anim:frame(0.0).scale

  anim:frame(99.0).position = vec2(start_position.x + offset, snow_settings.position.y + snow_settings.scale.y)
  anim:frame(99.0).opacity = anim:frame(0.0).opacity
  anim:frame(99.0).scale = anim:frame(0.0).scale

  anim:init(util.random_float(snow_settings.speed_range.x, snow_settings.speed_range.y), util.random_float(snow_settings.delay_range.x, snow_settings.delay_range.y))
  table.insert(self.animators, anim)
end

return snow

--[[ Example use case

local snow = require('snow')
local snow_effect = snow:new()

util.create_thread(function()
  snow_effect:update()
  snow_effect:render(vec2(0.0, 0.0), game.resolution())
end

]]--